# Bluecheese

This theme is for Drupal.org sites.

It is assumed you will be developing this theme specifically with the drupalorg project and its related infrastructure as documented below.

### Requirements

* You will need [DDEV](https://ddev.readthedocs.io/en/stable/).
* If you haven't installed mkcert before, you might need to do it as a one-off.
  See information here.

## Contributing to Drupalorg Bluecheese

Please see contributing instructions at https://www.drupal.org/docs/develop/git/using-gitlab-to-contribute-to-drupal/introduction


## Getting Started

The drupalorg project repo: <https://gitlab.com/drupal-infrastructure/sites/drupalorg>

For an optimal local development setup, it is recommended that you develop on the Bluecheese theme in combination with development on the Drupalorg site using the following steps.

### Local setup:

- `git clone git@gitlab.com:drupal-infrastructure/sites/drupalorg.git drupalorg && cd drupalorg`

- Create a `settings.local.php` file

- In your local settings file, you will need to do add these lines:

- `$settings['container_yamls'][] = DRUPAL_ROOT . '/sites/development.services.yml';`

- `$settings['cache']['bins']['discovery'] = 'cache.backend.null';`

- run `ddev start`

- run `ddev composer install`
- Note that running the above command can checkout the Bluecheese repo to the `2.x` branch and you may need to checkout your feature branch again after running this.

- Outside ddev run  `composer recreate`

- Please consult the project’s [README](https://gitlab.com/drupal-infrastructure/sites/drupalorg/-/blob/main/README.md?ref_type=heads) file for more information.


#### About composer recreate:

- Notice that all the action in the install process is happening with the “composer recreate” script.

- The fie: `scripts/composer/recreate.sh`

- It is currently set up to:

  - drop the database

  - install Drupal with the existing config
  - install test content

  - create a user login link that directs the user to an admin page offering a way to import some menu links. This functionality comes from[ the Structure Sync contrib module](https://www.drupal.org/project/structure_sync). (See Appendix below for more information)

### Compiling SASS
BLuecheese is in the process of an upgrade to remove SASS dependency but for now you do need to run gulp to compile any SASS updates by following the steps below:

1. run `ddev ssh`
1. Note that the required version of Node and Yarn are installed inside ddev
1. run `cd web/themes/contrib/bluecheese/`
1. run `yarn install`.
1. run `yarn compile` to compile Sass to CSS.
1. Run `yarn watch` to compile Sass on the fly as you update scss files.

Note that you can run any of these steps outside of ddev by adding `ddev` to the beginning of the command. Either way,
you encounter errors if you don't run `yarn` commands against `ddev`.

# Technical Approach

## Default Content

The project uses default content for providing a set of test content to run automated tests against.

The config for installing custom default content lives in a submodule inside the drupalorg contrib module at https://git.drupalcode.org/project/drupalorg

There are currently two default content modules.

- `drupalorg_test_content` is for local testing of CSS, JS and other site features, including automated testing.
- `drupalorg_demo_content` is for prototyping landing page content. Unless you are working specifically on content prototyping, it's safe to ignore this module.

Installing both modules at once is not recommended.

The `drupalorg_test_content` module provides test content for local development and to run automated tests against.

- The test content is meant to be representative of common pages on drupal.org.

- The module creates test blocks, nodes, taxonomy terms, media, and files in order to allow developers to test site functionality.
- To install test content, run `composer test-content`.

- Test content is installed automatically when you run `composer recreate`


### Exporting content to the Default Content module

- First, enable default_content module with `ddev drush en default_content`

Export content with Drush, using `ddev drush dcer`.

When you export content, you can optionally specify the entity type, entity id, and output location.

The dcer command will auto-export any referenced entities with your output.


#### Examples

- Export all un-exported nodes

  - `ddev drush dcer node`

- Export a specific node

  - `ddev drush dcer node 10`

- Export node 10 to the `drupalorg_test_content` module

  - `ddev drush dcer node 10 --folder=modules/contrib/drupalorg/modules/drupalorg_test_content/content`

The folder path is complex, so you can assume the --folder by running the provided composer command.

- `composer dcer node`

- `composer dcer node 10`



**Notes**

- When creating test content it is best to add a URL alias. Node ids are variable on import, so automated tests on Behat or Nightwatch will fail without aliases.

- Passing just the entity type will export new content only.

- Embedding media in a text field will not export.

- Exporting comments does not work as expected. See https://www.drupal.org/project/default_content/issues/3185327

- The Issues content type will be replaced by GitLab issues.

- The new Contribution records content type will store issue records.


## Theme Development

### Overview

- Theming for Drupalorg site will be iterating on the [Bluecheese](https://www.drupal.org/project/bluecheese) theme.

- Working principles:

  - Use a component based approach, and atomic design principles.

  - follow the lead of the design team on what components will be implemented, and how to structure the `components` directory.


### Set up to work on an issue

- When you spin up your local drupalorg site make sure you are checked out either to the `main` branch, or if your theme work requires changes to the main project, ensure that you are checked out to your drupalorg feature branch.

- Go to the theme directory and set up your local theme branch:

  * `cd web/themes/contrib/bluecheese`
  * If you already have a branch for your issue you can check it out here.
  * If you have not created an issue against the Bluecheese project you will need to do that before you can create an issue fork.
  * Please follow the standard [Contributor Issue fork workflow](https://www.drupal.org/docs/develop/git/using-gitlab-to-contribute-to-drupal/creating-issue-forks) to create issues and issue forks and branches on Bluecheese.


## Storybook

- We are using [Storybook](https://storybook.js.org/) to build the styleguide.

- We are using Yarn 4 to install, manage, and run Storybook.

- Storybook is integrated with Drupal using the [Drupal Storybook contrib module](https://www.drupal.org/project/storybook).

  - We use a Webpack5 server type installation


### Install and start Storybook

- To install storybook:

  - `composer install-storybook` (outside of ddev).
  - Note that this builds storybook .json files from the stories twig files one time. If you want to continuously build/update the .json files you can use the watch command below.

- To watch stories
  - run `ddev watch-stories` and the stories twig files will continue to export to json while you work so that Storybook will be updated automatically. This is the same as running `ddev drush storybook:generate-all-stories`

- To start Storybook:

  - `ddev yarn storybook`
    (I prefer running this in a separate tab, so I can work without constantly stopping and starting it.)

- Some updates you make during component development may require restarting Storybook.

- Clearing cache:
  - note that Drupal caching impacts storybook, so you will sometimes need to run `ddev drush cr`

- To restart Storybook:

  - `control + C`

  - `ddev yarn storybook`

For information on how we get our components to appear in Storybook, see the [Writing Stories ](#writing-stories)section below.


## Building Storybook components

- We _allow_ SDCs but do not _require_ them

  - There are known render issues that we have with SDC and Drupal theme functions; until those are solved, we can’t create a fully SDC theme.
  - There are also instances where the Style Guide is useful for showing markup that does not need to be tightly coupled to SDC architecture, particularly for showing base markup and color palettes.
  - Separately there are still instances when twig includes are useful outside of SDC structure.

- Despite that, a convenient way to build a component is to use the `drush generate sdc` command and adapt the boiler plate as needed. We can then either delete the `components.yml` file, or not define any props, and allow that to be work that happens later.

  - Note that whether or not a component is an SDC changes the syntax one uses to pull the component template into other components, or into Drupal templates.

- Twig templates should

  - utilize twig blocks

  - be compatible with Drupal attributes

  - have intuitively named variables

  - Using the guidelines and tools from the storybook module, create a stories.twig file

- Components in Storybook should get rendered with realistic test content.

- It should look and feel like a component you might actually see on Drupal.org.


## Building Drupal Components

The general approach for landing page components should be to use a paragraph as the base, top level component entity, and to provide all the component wide variables and properties and structure. Sub-Paragraphs are used for sub-components that are repeated elements, and support any variants on those components needed.

For other content types, implementation should be determined per instance. It may make sense to implement some fields on the content types as SDC components or just theme them directly on the content type,


### How to build drupal paragraph components

1. Field names should include the component name for paragraphs and custom blocks.
1. The actual field name after the naming convention should be aligned with the SDC field names for variable/properties.
1. Help text should be straightforward and work to match the field up with the SDC property’s acceptable values.
1. A paragraph type should be used inside the top level paragraph as a reference field where there is a repeated, regular structure.

1. Component naming should follow the same naming convention as the design in storybook and should be fairly seamless to identify in exported config / templates.

1. Fields required in SDC should be required in the paragraphs that implement them.


## Writing Stories

Some commands are required to build Stories for consumption by Storybook. Use the following as a guide:

- After editing a `component.stories.twig` file, run

`ddev drush storybook:generate-all-stories`

to regenerate the `component.stories.json` file for Storybook. Do not manually edit the JSON file.

- You may also run `ddev watch-stories` to auto-compile changes.

* After editing a `component.twig` file, you may need to run `ddev drush cr` to clear the Twig render cache.


## Example components
There are example SDC components with Storybook stories at `web/themes/contrib/bluecheese/components/examples`  to show example file structure and examples of syntax for the story twig files as well as component yml files.

See `web/themes/contrib/bluecheese/components/examples/structure/structure.component.yml` for examples of various field types as properties in the component yml of an SDC component.

## CSS naming conventions
Please follow [Block-Element-Modifier (BEM)](https://css-tricks.com/bem-101/) for component CSS class naming convention.

Some examples given component, `my-component`
- Use double hyphens for variant: `my-component--large`
- Use double underscore for child element `my-component__title`
- Use a combination when needed: `my-component__title--featured`
- When possible, try to architect components to not require double-nested variants or child elements.

See [Drupal Docs](https://www.drupal.org/docs/develop/standards/css/css-architecture-for-drupal-9) for more guidance on CSS architecture.

## Theme Testing

### Stylelint
The project is configured to run stylelint.

- To lint css files run `yarn lint-css`
- Many errors are automatically fixable. To fix stylelint errors add `--fix` to the end of the command.
- e.g. `yarn lint-css --fix`


## Accessibility Testing

We are using the [Storybook a11y add-on](https://storybook.js.org/addons/@storybook/addon-a11y) within the styleguide to test accessibility on components. You can find this if you visit any story and look at its control panel

- You will see an “Accessibility” tab:

- Click that tab for a list of the tests that were run on the component as well as the results

- Failing tests will offer explanations and recommendations for fixes

The project is set up to run automated accessibility tests with Nightwatch. Nightwatch runs Axe. To run Nightwatch do the following steps.

In the main project root:
- run `ddev yarn install`
- If you don't already have test content installed, run `composer test-content`
- create a `.env` file in your local project root and paste in the contents of the `.env.example` file in the project root.
- run `composer nightwatch`
- The test should run successfully on all the pages specified in the test: tests/Nightwatch/Tests/a11yTest.js
- Note the test will fail as there are actual accessibility issues with the current state of the theme.


