(function(Drupal) {

  Drupal.behaviors.tabContainer = {
    attach(context) {

    // tabs from https://codepen.io/kevinpowell/pen/oNQgRKm
    // updated to use BEM classes, and for multiple tabs containers on the page
    const tabsContainers = document.querySelectorAll(".tabs_container");
    tabsContainers.forEach((tabsContainer, indexList) => {
      tabsContainers[indexList].tabsList = tabsContainer.querySelector("ul");
      tabsContainers[indexList].tabButtons = tabsContainers[indexList].tabsList.querySelectorAll("a");
      tabsContainers[indexList].tabPanels = tabsContainer.querySelectorAll(".tabs_container__panels > div");

      tabsContainers[indexList].tabsList.setAttribute("role", "tablist");

      tabsContainers[indexList].tabsList.querySelectorAll("li").forEach((listitem) => {
        listitem.setAttribute("role", "presentation");
      });

      tabsContainers[indexList].tabButtons.forEach((tab, index) => {
        tab.setAttribute("role", "tab");
        if (index === 0) {
          tab.setAttribute("aria-selected", "true");
          // we'll add something here
        } else {
          tab.setAttribute("tabindex", "-1");
          tabsContainers[indexList].tabPanels[index].setAttribute("hidden", "");
        }
      });

      tabsContainers[indexList].tabPanels.forEach((panel) => {
        panel.setAttribute("role", "tabpanel");
        panel.setAttribute("tabindex", "0");
      });

      tabsContainers[indexList].tabsList.addEventListener("click", (e) => {
        const clickedTab = e.target.closest("a");
        if (!clickedTab) return;
        e.preventDefault();

        switchTab(clickedTab, indexList);
      });

      tabsContainer.addEventListener("keydown", (e) => {
        switch (e.key) {
          case "ArrowLeft":
            moveLeft(indexList);
            break;
          case "ArrowRight":
            moveRight(indexList);
            break;
          case "Home":
            e.preventDefault();
            switchTab(tabsContainers[indexList].tabButtons[0], indexList);
            break;
          case "End":
            e.preventDefault();
            switchTab(tabsContainers[indexList].tabButtons[tabsContainers[indexList].tabButtons.length - 1], indexList);
            break;
        }
      });

    });

    function moveLeft(index) {
      const currentTab = document.activeElement;
      if (!currentTab.parentElement.previousElementSibling) {
        switchTab(tabsContainers[index].tabButtons[tabsContainers[index].tabButtons.length - 1], index);
      } else {
        switchTab(
          currentTab.parentElement.previousElementSibling.querySelector("a"), index
        );
      }
    }

    function moveRight(index) {
      const currentTab = document.activeElement;
      if (!currentTab.parentElement.nextElementSibling) {
        switchTab(tabsContainers[index].tabButtons[0], index);
      } else {
        switchTab(currentTab.parentElement.nextElementSibling.querySelector("a"), index);
      }
    }

    function switchTab(newTab, index) {
      const activePanelId = newTab.getAttribute("href");
      const activePanel = document.querySelector(activePanelId);

      tabsContainers[index].tabButtons.forEach((button) => {
        button.setAttribute("aria-selected", false);
        button.setAttribute("tabindex", "-1");
      });

      tabsContainers[index].tabPanels.forEach((panel) => {
        panel.setAttribute("hidden", true);
      });

      activePanel.removeAttribute("hidden", false);

      newTab.setAttribute("aria-selected", true);
      newTab.setAttribute("tabindex", "0");
      newTab.focus();
    }

    },
  };

})(Drupal);
