(function(Drupal) {

  Drupal.behaviors.copyCodeBlock = {
    attach(context) {
      const copyButtonLabel = "Copy Code";

      // use a class selector if available
      let blocks = document.querySelectorAll(".code_wrapper");

      blocks.forEach((block) => {
      // only add button if browser supports Clipboard API
      if (navigator.clipboard) {
        if(block.classList.contains('copy_button') === false){
          let button = document.createElement("button");
          let innerButton = document.createElement("span");

          //hide the text but keep it findable for screenreaders
          innerButton.classList.add("visually-hidden");

          innerButton.innerText = copyButtonLabel;
          button.appendChild(innerButton);
          button.classList.add("icon-frame");
          button.classList.add("code_button");

          //mark this block as button applied, in case this js fires more than once
          block.classList.add("copy_button");
          block.appendChild(button);
          button.addEventListener("click", async () => {
            await copyCode(block, innerButton);
          });
        }
      }
      });

      async function copyCode(block, button) {
      let code = block.querySelector("code");
      let text = code.innerText;

      await navigator.clipboard.writeText(text);

      // visual feedback that task is completed
      button.innerText = "Copied!";
      button.classList.remove("visually-hidden");

      setTimeout(() => {
        button.innerText = copyButtonLabel;
        button.classList.add("visually-hidden");
      }, 700);
      }
    },
  };

})(Drupal);
