(function ($) {
  $(document).ready(function() {
    let windowsize = $(window).width();
    let contentWidth = $('#content').width();
    let machineName = $('#banner').find('.cta a').attr('class');
    let storage = localStorage.getItem(machineName);

    if (storage !== 'closed') {
      $("#banner").css('display', 'block').prepend( "<a class='close'>Close</a>" );
      $(".close").click(function() {
        $('#banner').hide();
        localStorage.setItem(machineName, 'closed');
      });
    }

    if ($('body.node-type-guide').length || $('body.node-type-documentation').length) {
      $("#tabs").addClass('js');
      $("#tabs ul li:first-child").prepend( "<span class='toggle'>Toggle</span>" );
      $(".toggle").click(function() {
        $('#tabs').toggleClass('clicked')
      });
    }

    // Get started menu dropdown
    const getStarted = $('#block-bluecheese-secondarymenu > ul > li > .button');
    const secondaryMenu = $('#block-bluecheese-secondarymenu');
    const secondaryMenuChildren = $('#block-bluecheese-secondarymenu > ul > li > ul');
    secondaryMenu.on("mouseenter", function() {
      secondaryMenu.addClass('expanded');
      secondaryMenuChildren.addClass('show');
      getStarted.addClass('open');
    }).on("mouseleave", function() {
      secondaryMenu.removeClass('expanded');
      secondaryMenuChildren.removeClass('show');
      getStarted.removeClass('open');
    });

    // Main navigation menu accessibility.
    $('#navigation-inner .button > a, #navigation-inner .expanded > a:first-child').attr('aria-expanded', 'false').focus(function (e) {
      // Delay to reduce double-triggering with click event.
      setTimeout(function () {
        e.target.parentElement.classList.add('menu-expanded');
        e.target.setAttribute('aria-expanded', 'true');
      }, 100);
    }).click(function (e) {
      e.target.parentElement.classList.toggle('menu-expanded');
      e.target.setAttribute('aria-expanded', e.target.parentElement.classList.contains('menu-expanded') ? 'true' : 'false');
      e.preventDefault();
    }).parent().find('a').addBack().blur(function () {
      // Wait for next element to gain focus.
      setTimeout(function () {
        let trigger = document.querySelector('#navigation-inner .menu-expanded');
        // Check if focused element is still in the expanded menu.
        if (trigger !== null && !trigger.contains(document.activeElement)) {
          trigger.classList.remove('menu-expanded');
          trigger.querySelector(':scope > a:first-child').setAttribute('aria-expanded', 'false');
        }
      }, 10);
    });

    // Click to expand search.
    const searchForm = document.querySelector('#block-search-form form');
    if (searchForm !== null) {
      searchForm.addEventListener('submit', function (e) {
        const header = this.closest('#nav-header');
        if (!header.classList.contains('search-expanded')) {
          e.preventDefault();
          header.classList.add('search-expanded');
          setTimeout(function () {
            searchForm.querySelector('input[type="text"]').focus();
          });
        }
      })
      searchForm.addEventListener('blur', function () {
        // Wait for next element to gain focus.
        setTimeout(function () {
          let trigger = document.querySelector('#nav-header.search-expanded');
          // Check if focused element is still in the expanded menu.
          if (trigger !== null && !trigger.querySelector('#search-block-form').contains(document.activeElement)) {
            trigger.classList.remove('search-expanded');
          }
        }, 150);
      }, true);
      // Skip to search a11y link.
      document.querySelector('.skip-link-search').addEventListener('click', function (e) {
        searchForm.querySelector('input[name="submit"]').click();
        e.preventDefault();
      });
    }

    // Accordion.
    document.querySelectorAll('.accordion > h3').forEach(function (header) {
      let content = header.nextElementSibling;
      let id = header.getAttribute('id');
      let contentId = id + '-content';

      header.parentNode.classList.add('accordion-processed');
      content.setAttribute('id', contentId);
      content.setAttribute('hidden', '');
      header.innerHTML = '<button aria-expanded="false" aria-controls="' + Drupal.checkPlain(contentId) + '">' + header.innerHTML + '</button>';
      header.querySelector('button').addEventListener('click', function (e) {
        if (e.target.getAttribute('aria-expanded') === 'true') {
          e.target.setAttribute('aria-expanded', 'false');
          e.target.parentNode.nextElementSibling.setAttribute('hidden', '');
        }
        else {
          e.target.setAttribute('aria-expanded', 'true');
          e.target.parentNode.nextElementSibling.removeAttribute('hidden');
          history.replaceState(null, null, '#' + e.target.parentNode.getAttribute('id'));
        }
      });

      // Expand if linked to with hash.
      if (window.location.hash === '#' + id) {
        header.querySelector('button').click();
      }
    });

    // See more (org pages for now)
    const collapsibles = Array.from(document.querySelectorAll('.drupalorg-inline-collapsible'));
    for (var i = 0; i < collapsibles.length; i++) {
      const seeMore = document.createElement('button');
      seeMore.classList.add('see-more');
      collapsibles[i].insertAdjacentElement('beforeend', seeMore);
      seeMore.addEventListener('click', function (e) {
        const classes = e.target.previousSibling.classList;
        if (classes.contains('collapsed')) {
          this.innerHTML= Drupal.t(" - See less");
          classes.remove('collapsed');
        }
        else {
          this.innerHTML = Drupal.t(" + See more");
          classes.add('collapsed');
        }
      });
      seeMore.click();
    }
    // Add container around tables to allow them to scroll horizontally when viewport is narrower than the table width.

    function wrapTable(table) {
      // Create the wrapper div with class.
      const newWrapper = document.createElement('div');
      newWrapper.classList.add('js-table--scrollable');

      // Insert the wrapper before the table
      table.parentNode.insertBefore(newWrapper, table);

      // Move the table into the wrapper
      newWrapper.appendChild(table);
    }

    // Get all the tables on page to loop through.
    const tables = document.querySelectorAll('table');

    // check if page is one where tables should not have horizontal scroll container added.
    let skipTable = false;

    // List classes on pages where we don't want tables to be scrollable.
    const exceptionSelectors = [".node--type-contribution-record", ".user-form"];

    exceptionSelectors.forEach(exceptionSelectors => {
      const element = document.querySelector(exceptionSelectors);
      if (element) {
        let skipTable = true;
      }
    });

    if (skipTable === false) {
      // Loop through each table and apply the wrapper div.
      for (let i = 0; i < tables.length; i++) {
        const table = tables[i];
        wrapTable(table);
      }
    }
  });
})(jQuery);
